"""
封装http请求函数
"""

import requests
from smssdk.utils.exceptions import SMSServiceException
from smssdk.utils.logger import Logger

logger = Logger('https.requests.abstract_func.py')
# 隐藏跳过ssl证书验证的警告
requests.packages.urllib3.disable_warnings()


def get(url: str, query_params: dict = None, headers: dict = None):
    """
    get请求
    :param url: 请求地址
    :param query_params: query参数
    :param headers: header参数
    :return: Object
    zgl/2022.09.08
    """
    # url校验
    if not url or type(url) is not str or str.strip(url) == '':
        raise SMSServiceException(f'[https.requests.abstract_func.get()]请求地址url有误，url:{url}')
    # 拼接真实url
    true_url = url
    if query_params:
        if type(query_params) is not dict:  # 对于集合类型，if val的结果：{}、[]、()都为False，故此处不需验证元素是否为空
            raise SMSServiceException(
                f'[https.requests.abstract_func.get()]函数收到了query_params，但其值有误，query_params:{query_params}')
        else:
            true_url += '?'
            for one_key in query_params.keys():
                true_url = true_url + str(one_key) + '=' + str(query_params[one_key]) + '&'
            # 去掉末尾多于的'&'
            true_url = true_url[:-1]
    # headers校验
    if headers and type(headers) is not dict:
        raise SMSServiceException(f'[https.requests.abstract_func.get()]函数收到了headers，但其值有误，headers:{headers}')
    # 发起请求
    try:
        if headers:
            response = requests.get(url=true_url, headers=headers, verify=False)
        else:
            response = requests.get(url=true_url, verify=False)
    except Exception as e:
        logger.exception(f'[get()]请求{true_url}异常。', e)
        raise SMSServiceException(f'[https.requests.abstract_func.get()]请求{true_url}异常')
    # 解析响应
    try:
        return response.text
    except Exception as e:
        logger.exception(f'[get()]解析{true_url}的响应结果异常。', e)
        raise SMSServiceException(f'[https.requests.abstract_func.get()]解析{true_url}的响应结果异常')


def post(url: str, query_params: dict = None, body_params: any = None, headers: dict = None):
    """
    post请求
    zgl/2022.09.08
    :param url: 请求地址
    :param query_params: query参数
    :param body_params: body参数
    :param headers: header参数
    :return: Object
    """
    # url校验
    if not url or type(url) is not str or str.strip(url) == '':
        raise SMSServiceException(f'[https.requests.abstract_func.post()]请求地址url有误，url:{url}')
    # 拼接真实url
    true_url = url
    if query_params:
        if type(query_params) is not dict:  # 对于集合类型，if val的结果：{}、[]、()都为False，故此处不需验证元素是否为空
            raise SMSServiceException(f'[https.requests.abstract_func.post()]函数收到了query_params，但其值有误，query_params:{query_params}')
        else:
            true_url += '?'
            for one_key in query_params.keys():
                true_url = true_url + str(one_key) + '=' + str(query_params[one_key]) + '&'
            # 去掉末尾多于的'&'
            true_url = true_url[:-1]
    # body校验
    if not body_params or (type(body_params) is not dict and type(body_params) is not list):
        raise SMSServiceException(f'[https.requests.abstract_func.post()]函数收到了body_params，但其值有误，body_params:{body_params}')
    # headers校验
    if headers and type(headers) is not dict:
        raise SMSServiceException(f'[https.requests.abstract_func.post()]函数收到了headers，但其值有误，headers:{headers}')
    # 发起请求
    try:
        if headers:
            if body_params:
                response = requests.post(url=true_url, json=body_params, headers=headers, verify=False)
            else:
                response = requests.post(url=true_url, headers=headers, verify=False)
        else:
            if body_params:
                response = requests.post(url=true_url, json=body_params, verify=False)
            else:
                response = requests.post(url=true_url, verify=False)
    except Exception as e:
        logger.exception(f'[post()]请求{true_url}异常。', e)
        raise SMSServiceException(f'[abstract_func:post()]请求{true_url}异常')
    # 解析响应
    try:
        return response.text
    except Exception as e:
        logger.exception(f'[post()]解析{true_url}的响应结果异常。', e)
        raise SMSServiceException(f'[https.requests.abstract_func.post()]解析{true_url}的响应结果异常')



"""
token获取和过期处理
"""

import time
from smssdk.utils.logger import Logger
from smssdk.https.requests.api_request import request_for_token
from smssdk.utils.exceptions import SMSServiceException

logger = Logger('https.auth.auth.py')
# 存放token
token = ''
# 当前token下次过期时间
next_expire_time = 0


def get_token(account: str, password: str):
    """
    返回一个必定有效的token
    zgl/2022.09.08
    :param: account 账号
    :param: password 密码
    :return: token
    """
    if not account or type(account) is not str or str.strip(account) == '':
        raise SMSServiceException(f'[https.auth.auth.get_token()]入参account不存在，account:{account}')
    if not password or type(password) is not str or str.strip(password) == '':
        raise SMSServiceException(f'[https.auth.auth.get_token()]入参password不存在，password:{password}')
    global token
    global next_expire_time
    # token不存在则获取
    if not token or str.strip(token) == '':
        try:
            token, next_expire_time = request_for_token(account, password)
        except SMSServiceException as e:
            raise e
    # 已存在的token
    else:
        current_time = time.time()
        current_timestamp = int(round(current_time * 1000))
        # 已过期则重新获取
        if current_timestamp >= next_expire_time:
            try:
                token, next_expire_time = request_for_token(account, password)
            except SMSServiceException as e:
                raise e
    return token



"""
回调响应封装
"""
from smssdk.utils.values import CALLBACK_CODE


def callback_success(data: dict = None) -> dict:
    res = CALLBACK_CODE['SUCCESS']
    res['data'] = data
    return res


def callback_failed(data: dict = None) -> dict:
    res = CALLBACK_CODE['FAILED']
    res['data'] = data
    return res



"""
解析SP->ISMG连接请求中ISMG响应的消息体
"""
import struct

from . import ResponseData
from smssdk.utils.logger import Logger

logger = Logger('tcp.cmpp.resdata.connect_res.py')


class ConnectRes(ResponseData):
    def __init__(self, message):
        # logger.info(f'[ConnectRes.__init__()]收到连接ISMG网关的响应，消息参数：{message.hex()}')

        # 解析出消息头和消息体
        ResponseData.__init__(self, message)
        message_body = self.message_body

        # 解析消息体
        # 状态Status
        self.status, = struct.unpack('!B', message_body[0:1])
        # ISMG认证码AuthenticatorISMG，用于鉴别ISMG
        self.raw_authenticator_ISMG = message_body[1:17]
        # 服务器支持的最高版本号Version
        self.version, = struct.unpack('!B', message_body[17:])

        # 回调dict
        self.callback_res = {
            'status': self.status,
            'authenticator_ISMG': self.raw_authenticator_ISMG.hex(),
            'version': self.version,
        }



"""
解析ISMG网关响应入口
"""
import struct


class ResponseData:
    def __init__(self, message):
        self.length, = struct.unpack('!L', message[0:4])
        self.command_id, = struct.unpack('!L', message[4:8])
        self.sequence, = struct.unpack('!L', message[8:12])
        self.message_body = message[12:self.length]



"""
解析SP->ISMG发送消息事件中ISMG响应的消息体
"""

import struct
from smssdk.tcp.cmpp.resdata import ResponseData
from smssdk.utils.logger import Logger
from smssdk.utils.values import CMPP_VERSION

logger = Logger('tcp.cmpp.resdata.submit_res.py')


class SubmitRes(ResponseData):
    def __init__(self, message, version: int):
        """
        :param version: CMPP版本。2/3
        """
        # logger.info(f'[SubmitRes.__init__()]收到向ISMG网关发送消息的响应，消息参数：{message.hex()}。')

        # 解析出消息头和消息体
        ResponseData.__init__(self, message)
        message_body = self.message_body

        # 解析消息体
        # Msg_Id
        self.msg_id, = struct.unpack('!Q', message_body[0:8])
        # Result。v2长度为1，v3长度为4
        if version == CMPP_VERSION['TWO']:
            self.result, = struct.unpack('!B', message_body[8:])
        else:
            self.result, = struct.unpack('!L', message_body[8:])

        self.callback_res = {
            'msg_id': self.msg_id,
            'result': self.result
        }



"""
请求http接口
"""

from smssdk.utils.values import URLS, RESPONSE_INFO, STABLE_HEADERS
from smssdk.utils.exceptions import SMSServiceException
from smssdk.https.requests.abstract_func import post
import json


def request_for_token(account: str, password: str):
    """
    请求获取token
    zgl/2022.09.08
    :return: token, expireTime
    """

    # 参数校验
    if not account or type(account) is not str or str.strip(account) == '':
        raise SMSServiceException(f'[https.requests.api_request.request_for_token()]入参account有误，account:{account}')
    if not password or type(password) is not str or str.strip(password) == '':
        raise SMSServiceException(f'[https.requests.api_request.request_for_token()]入参password有误，password:{password}')

    # body
    body = {
        'account': account,
        'password': password
    }
    # headers
    headers = {'Content-Type': STABLE_HEADERS['CONTENT_TYPE']}

    # 请求接口
    try:
        response = post(url=URLS['TOKEN'], body_params=body, headers=headers)
        json_res = json.loads(response)
    except SMSServiceException as e:
        raise e

    # 解析响应
    code_obj = json_res[RESPONSE_INFO['CODE_OBJ']]
    code = code_obj[RESPONSE_INFO['RESPONSE_CODE']]
    if code != RESPONSE_INFO['SUCCESS_CODE']:
        message = code_obj[RESPONSE_INFO['RESPONSE_MESSAGE']]
        raise SMSServiceException(
            f'[https.requests.api_request.request_for_token()]请求成功，但未取得预期结果，code:{code}，message:{message}')
    else:
        data = json_res[RESPONSE_INFO['DATA']]
        # expireTime为ms级时间戳
        return data['authenticationToken'], data['expireTime']


def request_for_batch_sms(token: str, phone_numbers: list, message_content: str, access_number: str = None):
    """
    发送短信（可一对多群发）
    zgl/2022.09.08
    :param token: token凭证
    :param phone_numbers: 目标电话号码
    :param message_content: 短信内容
    :param access_number: 虚拟接入码
    :return: 短信发送记录，[{'phoneNumber': str, 'msgId': str, 'code': int}, ...]
    """
    # 参数校验
    if not token or type(token) is not str or str.strip(token) == '':
        raise SMSServiceException(f'[https.requests.api_request.request_for_batch_sms()]token有误，token:{token}')
    if not phone_numbers or type(phone_numbers) is not list:
        raise SMSServiceException(
            f'[https.requests.api_request.request_for_batch_sms()]phone_numbers有误，phone_numbers:{phone_numbers}')
    if not message_content or type(message_content) is not str or str.strip(message_content) == '':
        raise SMSServiceException(f'[https.requests.api_request.request_for_batch_sms()]message_content有误，'
                                  f'message_content:{message_content}')

    # 封装headers
    headers = {
        RESPONSE_INFO['AUTH_NAME']: token,
        'Content-Type': STABLE_HEADERS['CONTENT_TYPE']
    }
    # 封装body
    body = {'phoneNumbers': phone_numbers, 'messageContent': message_content, 'accessNumber': access_number}

    # 请求接口
    try:
        response = post(url=URLS['BATCH_SMS'], body_params=body, headers=headers)
        json_res = json.loads(response)
    except SMSServiceException as e:
        raise e

    # 解析响应
    code_obj = json_res[RESPONSE_INFO['CODE_OBJ']]
    code = code_obj[RESPONSE_INFO['RESPONSE_CODE']]
    if code != RESPONSE_INFO['SUCCESS_CODE']:
        message = code_obj[RESPONSE_INFO['RESPONSE_MESSAGE']]
        raise SMSServiceException(
            f'[https.requests.api_request.request_for_batch_sms()]请求成功，但未取得预期结果，code:{code}，message:{message}')
    else:
        data = json_res[RESPONSE_INFO['DATA']]
        return data


def request_for_p2p_sms(token: str, params_list: list):
    """
    发送短信（批量点对点单发）
    zgl/2022.09.08
    :param token: token凭证
    :param params_list: sms参数列表，结构如下：
        [
            {
                'phoneNumber': str,
                'messageContent': str,
                'bizId': str,
                'accessNumber': str
            },
            ...
        ]
    :return: 短信发送记录，[{'phoneNumber': str, 'msgId': str, 'code': int}, ...]
    """
    # 参数校验
    if not token or type(token) is not str or str.strip(token) == '':
        raise SMSServiceException(f'[https.requests.api_request.request_for_p2p_sms()]token有误，token:{token}')
    if not params_list or type(params_list) is not list:
        raise SMSServiceException(
            f'[https.requests.api_request.request_for_p2p_sms()]params_list有误，params_list:{params_list}')
    else:
        for x in params_list:
            if not x['phoneNumber'] or type(x['phoneNumber']) is not str or str.strip(x['phoneNumber']) == '':
                raise SMSServiceException(
                    f"[https.requests.api_request.request_for_p2p_sms()]params_list存在某元素的phoneNumber有误, "
                    f"phoneNumber:{x['phoneNumber']}")
            if not x['messageContent'] or type(x['messageContent']) is not str or str.strip(x['messageContent']) == '':
                raise SMSServiceException(
                    f"[https.requests.api_request.request_for_p2p_sms()]params_list存在某元素的messageContent有误, "
                    f"messageContent:{x['messageContent']}")

    # 封装headers
    headers = {
        RESPONSE_INFO['AUTH_NAME']: token,
        'Content-Type': STABLE_HEADERS['CONTENT_TYPE']
    }

    # 请求接口
    try:
        response = post(url=URLS['P2P_SMS'], body_params=params_list, headers=headers)
        json_res = json.loads(response)
    except SMSServiceException as e:
        raise e

    # 解析响应
    code_obj = json_res[RESPONSE_INFO['CODE_OBJ']]
    code = code_obj[RESPONSE_INFO['RESPONSE_CODE']]
    if code != RESPONSE_INFO['SUCCESS_CODE']:
        message = code_obj[RESPONSE_INFO['RESPONSE_MESSAGE']]
        raise SMSServiceException(
            f'[https.requests.api_request.request_for_p2p_sms()]请求成功，但未取得预期结果，code:{code}，message:{message}')
    else:
        data = json_res[RESPONSE_INFO['DATA']]
        return data



"""
解析SP->ISMG断开连接请求中ISMG响应的消息体
"""
from smssdk.tcp.cmpp.resdata import ResponseData
from smssdk.utils.logger import Logger

logger = Logger('tcp.cmpp.resdata.terminate_res.py')


class TerminateRes(ResponseData):
    def __init__(self, message):
        # logger.info(f'[TerminateRes.__init__()]收到断开ISMG网关连接的响应，消息参数：{message.hex()}。')

        # 拆除ISMG连接响应无消息体
        ResponseData.__init__(self, message)



"""
SP->ISMG断开连接参数封装
"""

from smssdk.tcp.cmpp.reqdata import RequestData
from smssdk.utils.values import CMPP_COMMAND_ID


class TerminateReq(RequestData):
    def __init__(self):
        # 拆除ISMG连接无消息体
        message_body = b''
        RequestData.__init__(self, CMPP_COMMAND_ID['CMPP_TERMINATE'], message_body)



"""
ISMG响应解析函数路由
"""
import struct

from smssdk.tcp.cmpp.resdata.connect_res import ConnectRes
from smssdk.tcp.cmpp.resdata.deliver_req import DeliverReq
from smssdk.tcp.cmpp.resdata.submit_res import SubmitRes
from smssdk.tcp.cmpp.resdata.terminate_res import TerminateRes
from smssdk.tcp.cmpp.resdata.active_req import ActiveReq
from smssdk.utils.values import CMPP_COMMAND_ID

# 不需要根据cmpp版本区分的解析函数
_res_mapping_common = {
    CMPP_COMMAND_ID['CMPP_CONNECT_RESP']: ConnectRes
    , CMPP_COMMAND_ID['CMPP_TERMINATE_RESP']: TerminateRes
    , CMPP_COMMAND_ID['CMPP_ACTIVE_TEST']: ActiveReq
}

# 需要根据cmpp版本区分的解析函数
_res_mapping_diff_by_version = {
    CMPP_COMMAND_ID['CMPP_SUBMIT_RESP']: SubmitRes
    , CMPP_COMMAND_ID['CMPP_DELIVER']: DeliverReq
}


def parse_response(message, version: int):
    """
    路由具体的解析函数
    :param message: 原生的ISMG响应
    :param version: CMPP版本
    :return:
    """
    command_id, = struct.unpack('!L', message[4:8])
    if command_id in _res_mapping_common.keys():
        return _res_mapping_common[command_id](message)
    elif command_id in _res_mapping_diff_by_version.keys():
        return _res_mapping_diff_by_version[command_id](message, version)



"""
解析ISMG->SP发送消息事件中ISMG的请求消息
"""
import struct

from smssdk.tcp.cmpp.resdata import ResponseData
from smssdk.utils.logger import Logger
from smssdk.utils.values import CMPP_REGISTERED_DELIVERY, CMPP_VERSION

logger = Logger('tcp.cmpp.resdata.deliver_req.py')


class DeliverReq(ResponseData):
    def __init__(self, message, version: int):
        # logger.info(f'[DeliverReq.__init__()]收到ISMG网关发送的消息，内容：{message.hex()}。')

        # 解析出消息头和消息体
        ResponseData.__init__(self, message)
        message_body = self.message_body

        # v2和v3不同解析
        if version == CMPP_VERSION['TWO']:
            # Registered_Delivery。长度1字节，无符号整数
            self.registered_delivery, = struct.unpack('!B', message_body[63:64])
            # 判断是不是状态报告，是的话解析状态报告（msg_content）
            if self.registered_delivery == CMPP_REGISTERED_DELIVERY['IS_REPORT']:
                self.is_report = True
                self.raw_report = message_body[65:125]
                self.callback_res = {
                    'msg_id': struct.unpack('!Q', self.raw_report[0:8])[0],
                    'stat': self.raw_report[8:15].decode('utf-8'),
                    'submit_time': self.raw_report[15:25].decode('utf-8'),
                    'done_time': self.raw_report[25:35].decode('utf-8'),
                    'dest_terminal_id': self.raw_report[35:56].decode('utf-8'),
                    'smsc_sequence': struct.unpack('!L', self.raw_report[56:])[0]
                }
                self.msg_id = self.callback_res['msg_id']
            else:
                self.is_report = False
        else:
            # Registered_Delivery。长度1字节，无符号整数
            self.registered_delivery, = struct.unpack('!B', message_body[75:76])
            # 判断是不是状态报告，是的话解析状态报告（msg_content）
            if self.registered_delivery == CMPP_REGISTERED_DELIVERY['IS_REPORT']:
                self.is_report = True
                self.raw_report = message_body[77:148]
                self.callback_res = {
                    'msg_id': struct.unpack('!Q', self.raw_report[0:8])[0],
                    'stat': self.raw_report[8:15].decode('utf-8'),
                    'submit_time': self.raw_report[15:25].decode('utf-8'),
                    'done_time': self.raw_report[25:35].decode('utf-8'),
                    'dest_terminal_id': self.raw_report[35:67].decode('utf-8'),
                    'smsc_sequence': struct.unpack('!L', self.raw_report[67:])[0]
                }
                self.msg_id = self.callback_res['msg_id']
            else:
                self.is_report = False



"""
sms tcp cmpp sdk
"""
import hashlib
import re
import struct
import time

from smssdk.tcp.cmpp.reqdata import connect_req, submit_req, terminate_req
from smssdk.utils.logger import Logger
from smssdk.utils.exceptions import SMSServiceException
from smssdk.utils.values import NUMBERS, REGEX, CMPP_VERSION, MESSAGE_SPLIT_MARK_PREFIX
from .callback import BasicCallback
from .sockettool import Socket
from smssdk.utils.mythread import Thread

logger = Logger('tcp.cmpp.__init__.py')


class CmppTool:
    def __init__(self, version: int, host: str, port: int, sp_id: str, sp_secret: str, access_number: str,
                 callback_imp: BasicCallback = None):
        """
        :param version: 版本。2/3
        :param host: 服务地址
        :param port: 端口。0~65535
        :param sp_id: 客户spId
        :param sp_secret: sp公钥
        :param access_number: 接入码
        :param callback_imp: 回调类，继承BasicCallback并重写方法
        """
        if version not in CMPP_VERSION.values():
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool.__init__()]入参version有误，仅可为2或3')
        # 3.0暂时不可用
        if version == CMPP_VERSION['THREE']:
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool.__init__()]version3.0暂不可用，请用2.0版本')
        if not host or type(host) is not str or str.strip(host) == '':
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool.__init__()]入参host有误，host:{host}')
        if not port or type(port) is not int or port < NUMBERS['ONE'] or port > NUMBERS['MAX_PORT']:
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool:__init__()]入参port有误，port:{port}')
        if not sp_id or type(sp_id) is not str or str.strip(sp_id) == '':
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool:__init__()]入参sp_id有误，sp_id:{sp_id}')
        if not sp_secret or type(sp_secret) is not str or str.strip(sp_secret) == '':
            raise SMSServiceException(
                f'[tcp.cmpp.__init__.py:CmppTool:__init__()]入参sp_secret有误，sp_secret:{sp_secret}')
        if not access_number or type(access_number) is not str or str.strip(access_number) == '':
            raise SMSServiceException(
                f'[tcp.cmpp.__init__.py:CmppTool:__init__()]入参access_number有误，access_number:{access_number}')

        self._cmpp_version = version
        self._sp_id = sp_id
        self._sp_secret = sp_secret
        self._access_number = access_number
        self._callback = callback_imp
        if not self._callback:
            self._callback = BasicCallback()

        # 记录长短信拆分操作次数，作为该操作的唯一序号
        self._msg_split_key = NUMBERS['ZERO']

        # 初始化socket
        self._so = Socket(cmpp_version=self._cmpp_version, host=host, port=port, callback=self._callback)

    def connect_ismg(self):
        """
        连接ISMG，创建通道
        :return: bool。True：创建成功
        """
        # socket连接是否已在线
        if self._so.is_online():
            logger.info(
                f"[CmppTool.connect_ismg()][v{self._cmpp_version}]"
                f"连接socket服务端时检测到原有连接，将使用原有连接。")
            return
        # 连接socket
        self._so.connect_socket()
        # 启用子线程持续监听服务端发送的消息。函数作为参数时，不要写()，此处为read而非read()
        Thread(target=self._so.read).start()
        # SP->ISMG连接的请求参数
        connect_req_params = connect_req.ConnectReq(self._sp_id, self._sp_secret, version=self._cmpp_version)
        # 发出SP-ISMG的连接请求
        self._so.write(connect_req_params.message, event='connect_imsg')

    def terminate_ismg(self):
        """
        断开与ISMG的连接
        """
        # socket连接是否还在线
        if not self._so.is_online():
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool.terminate_ismg()]socket连接不存在或已不在线，操作无效。')
        # 请求断开ISMG
        self._so.write(terminate_req.TerminateReq().message, event='terminate_ismg')
        # 断开socket连接
        self._so.close_socket()

    def submit_msg(self, message: str, phone_numbers: list):
        """
        发送信息
        """
        # 目的号码格式校验
        if not phone_numbers or type(phone_numbers) is not list:
            raise SMSServiceException(
                f'[tcp.cmpp.__init__.py:CmppTool.submit_msg()]'
                f'目的号码phone_numbers有误，其类型需为list，且每个元素均为正确格式的手机号码。phone_numbers:{phone_numbers}')
        else:
            for x in phone_numbers:
                if not x or type(x) is not str or str.strip(x) == '' or not re.search(REGEX['MOBILE_PHONE'], x):
                    raise SMSServiceException(f"[tcp.cmpp.__init__.py:CmppTool.submit_msg()]phone_numbers存在某元素有误, "
                                              f"有误的phoneNumber:{x}")
        # 消息内容校验
        if message is None or type(message) is not str or str.strip(message) == '':
            raise SMSServiceException(
                f'[tcp.cmpp.__init__.py:CmppTool.submit_msg()]'
                f'消息内容message有误，其需为String类型。message:{message}')
        # socket连接是否还在线
        if not self._so.is_online():
            raise SMSServiceException(f'[tcp.cmpp.__init__.py:CmppTool.submit_msg()]socket连接不存在或已不在线，操作无效。')
        # 是否需要做长短信拆分
        _split_send = False
        _split_count = NUMBERS['ONE']
        if len(message) > NUMBERS['SEVENTY']:
            # 需要拆分
            _split_send = True
            # 计算拆分后的条数
            _remainder = len(message) % NUMBERS['SIXTY_SEVEN']
            if _remainder != NUMBERS['ZERO']:
                _split_count += len(message) // NUMBERS['SIXTY_SEVEN']
            else:
                _split_count = len(message) // NUMBERS['SIXTY_SEVEN']
        # SP->ISMG发送消息的请求参数（需要状态报告，故只能对每个号码单发）
        for x in phone_numbers:
            if _split_send:
                # 拆分次数+1。每对一个号码做一次长短信拆分业务，均视为单独的一组。
                self._msg_split_key += NUMBERS['ONE']
                logger.info(
                    f"[CmppTool.submit_msg()][v{self._cmpp_version}]"
                    f"即将做第{self._msg_split_key}组长短信拆分业务，目标手机号:{x}。")
                # 对于长短信，逐条发送拆分过的短信
                current_record = {
                    'message': {},
                    # 拆分消息总条数
                    'total_count': _split_count,
                    # 已收到响应的拆分消息成功条数。如果某条失败，则整体失败
                    'res_success_count': NUMBERS['ZERO'],
                    # 已收到ISMG状态报告成功的拆分消息条数。如果某条失败，则整体失败。
                    'reported_success_count': NUMBERS['ZERO'],
                    # 存放所有状态报告
                    'all_report': [],
                    # 上一次收到状态报告的时间。用于清除异常留存的拆分记录数据，防止内存泄漏。注册时间当作首首次接收时间。
                    'last_report_timestamp': int(time.time()),
                    # 主短信的seq_id
                    'main_seq_id': None,
                    # 主短信的msg_id
                    'main_msg_id': None,
                    # True标志着该组数据已经无用，待自动删除了
                    'invalid': False,
                    # 被标记为无用时的时间戳（秒级）
                    'invalid_timestamp': None
                }
                for i in range(_split_count):
                    _current_count = i + NUMBERS['ONE']
                    _msg_id = _current_count
                    _mark = MESSAGE_SPLIT_MARK_PREFIX + struct.pack('!L', _msg_id) + \
                            struct.pack('!L', _split_count) + struct.pack('!L', _current_count)
                    _content = ''
                    if _current_count == _split_count:
                        _content = message[NUMBERS['SIXTY_SEVEN'] * i:]
                    else:
                        _content = message[NUMBERS['SIXTY_SEVEN'] * i: _current_count * NUMBERS['SIXTY_SEVEN']]
                    obj_msg = submit_req.SubmitReq(
                        version=self._cmpp_version, msg_src=self._sp_id, msg_content=message, dest_terminal_id=[x],
                        src_id=self._access_number)
                    # 给主短信seq_id赋值
                    if i == NUMBERS['ZERO']:
                        current_record['main_seq_id'] = obj_msg.sequence
                    # 给current_record添加一条message数据
                    current_record['message'][_current_count] = {
                        # 该条消息的序列号
                        'seq_id': obj_msg.sequence,
                        # 要发送的具体消息
                        'message': obj_msg.message,
                        # 该条消息是否已收到服务器响应
                        'received_res': False,
                        # 收到的响应是成功才算业务成功
                        'service_success': False
                    }
                # 向socket消息监听事件注册该组长短信拆分发送记录
                self._so.registry_split_rec(self._msg_split_key, current_record)
                # SP-ISMG发送消息
                for current_count, value in current_record['message'].items():
                    self._so.write(value['message'], event=f'[split:{current_count}]submit_msg->{x}')
            else:
                submit_params = submit_req.SubmitReq(
                    version=self._cmpp_version, msg_src=self._sp_id, msg_content=message, dest_terminal_id=[x],
                    src_id=self._access_number)
                # SP-ISMG发送消息
                self._so.write(submit_params.message, event=f'submit_msg->{x}')



"""
sms http sdk
"""

from smssdk.https.auth.auth import get_token
from smssdk.https.requests.api_request import request_for_batch_sms, request_for_p2p_sms
from smssdk.utils.exceptions import SMSServiceException
from smssdk.utils.values import REGEX
import re


class HttpTool:
    def __init__(self, account: str, password: str):
        """
        :param account: 客户账号
        :param password: 客户账号密码
        """
        if not account or type(account) is not str or str.strip(account) == '':
            raise SMSServiceException(f'[https.__init__.py:HttpTool.__init__()]入参account有误，account:{account}')
        if not password or type(password) is not str or str.strip(password) == '':
            raise SMSServiceException(f'[https.__init__.py:HttpTool.__init__()]入参password有误，password:{password}')
        self._account = account
        self._password = password

    def _http_batch_sms(self, phone_numbers: list, message_content: str, access_number: str = None):
        """
        调用此函数发起sms批量短信发送服务
        zgl/2022.09.08
        :param phone_numbers: 目标手机号
        :param message_content: 短信内容
        :param access_number: 虚拟接入码。非必填
        :return: 短信发送记录，[{'phoneNumber': str, 'msgId': str, 'code': int}, ...]
        """
        # 入参校验
        if not phone_numbers or type(phone_numbers) is not list:
            raise SMSServiceException(
                f'[https.__init__.py:HttpTool.http_batch_sms()]入参phone_numbers有误，phone_numbers:{phone_numbers}')
        else:
            for x in phone_numbers:
                if not x or type(x) is not str or str.strip(x) == '' or not re.search(REGEX['MOBILE_PHONE'], x):
                    raise SMSServiceException(f"[HttpTool:http_batch_sms]phone_numbers存在某元素有误, phoneNumber:{x}")
        if not message_content or type(message_content) is not str or str.strip(message_content) == '':
            raise SMSServiceException(f'[https.__init__.py:HttpTool.http_batch_sms()]入参message_content有误，'
                                      f'message_content:{message_content}')
        # 获取token
        token = get_token(self._account, self._password)
        # 调用api
        return request_for_batch_sms(token, phone_numbers, message_content, access_number)

    def _http_p2p_sms(self, params_list: list):
        """
        调用此函数发起sms点对点短信发送服务
        zgl/2022.09.08
        :param params_list: sms参数列表，结构如下：
            [
                {
                    'phoneNumber': str,
                    'messageContent': str,
                    'bizId': str,
                    'accessNumber': str
                },
                ...
            ]
        :return: 短信发送记录，[{'phoneNumber': str, 'msgId': str, 'code': int}, ...]
        """
        # 入参校验
        if not params_list or type(params_list) is not list:
            raise SMSServiceException(
                f'[https.__init__.py:HttpTool.http_p2p_sms()]params_list有误，params_list:{params_list}')
        else:
            for x in params_list:
                if not x['phoneNumber'] or type(x['phoneNumber']) is not str or str.strip(x['phoneNumber']) == '' \
                        or not re.search(REGEX['MOBILE_PHONE'], x['phoneNumber']):
                    raise SMSServiceException(
                        f"[https.__init__.py:HttpTool.http_p2p_sms()]params_list存在某元素的phoneNumber有误, "
                        f"phoneNumber:{x['phoneNumber']}")
                if not x['messageContent'] or type(x['messageContent']) is not str or str.strip(
                        x['messageContent']) == '':
                    raise SMSServiceException(
                        f"[https.__init__.py:HttpTool.http_p2p_sms()]params_list存在某元素的messageContent有误, "
                        f"messageContent:{x['messageContent']}")
        # 获取token
        token = get_token(self._account, self._password)
        # 调用api
        return request_for_p2p_sms(token, params_list)



"""
SP->ISMG信息发送参数封装
"""

import struct
from smssdk.tcp.cmpp.reqdata import RequestData
from smssdk.utils.exceptions import SMSServiceException
from smssdk.utils.values import CMPP_COMMAND_ID, CMPP_VERSION


class SubmitReq(RequestData):
    def __init__(self,
                 version: int,
                 msg_src,
                 msg_content: str,
                 src_id: str,
                 dest_terminal_id: list = None,
                 pk_total=1,
                 pk_number=1,
                 registered_delivery: int = 1,
                 msg_level=0,
                 service_id='MI',
                 fee_usertype=2,
                 fee_terminal_id="",
                 tp_pid=0,
                 tp_udhi=0,
                 msg_fmt=8,
                 feetype='01',
                 feecode='103013',
                 valid_time=17 * '\x00',
                 at_time=17 * '\x00',
                 reserve='',
                 fee_terminal_type=0,
                 dest_terminal_type=0,
                 link_id=20 * '\x00',
                 split_send=False,
                 content_prefix: bytes = ''):
        """
        构建SP->ISMG发送消息事件的消息体
        :param version: cmpp协议版本，2/3
        :param msg_src: 即spId，由客户传入
        :param msg_content: 消息内容，客户传入
        :param src_id: 接入码。由客户传入
        :param dest_terminal_id: 接收短信的号码
        :param pk_total:
        :param pk_number:
        :param registered_delivery: 是否需要返回状态报告
        :param msg_level:
        :param service_id:
        :param fee_usertype:
        :param fee_terminal_id:
        :param tp_pid:
        :param tp_udhi:
        :param msg_fmt:
        :param feetype:
        :param feecode:
        :param valid_time:
        :param at_time:
        :param reserve: v2独有参数：保留。
        :param fee_terminal_type: v3独有参数：被计费用户的号码类型，0：真实号码；1：伪码。
        :param dest_terminal_type: v3独有参数：接收短信的用户的号码类型，0：真实号码；1：伪码。
        :param link_id: v3独有参数：点播业务使用的LinkID，非点播类业务的MT流程不使用该字段。
        :param split_send: 是否做了长短信拆分。True/False
        :param content_prefix: 长短信拆分后，要加在content前面的标志。
        """

        # 信息标识Msg_Id，由SP侧短信网关本身产生，本处填空。8字节。
        _msg_id = 8 * b'\x00'
        # 相同Msg_Id的信息总条数Pk_total，从1开始。1字节。
        _pk_total = struct.pack('!B', pk_total)
        # 相同Msg_Id的信息序号Pk_number，从1开始。1字节。
        _pk_number = struct.pack('!B', pk_number)
        # 是否要求返回状态确认报告Registered_Delivery，0：不需要；1：需要；2：产生SMC话单。
        _registered_delivery = struct.pack('!B', registered_delivery)
        # 信息级别Msg_level。1字节。
        _msg_level = struct.pack('!B', msg_level)
        # 业务类型Service_Id，是数字、字母和符号的组合。10字节。
        _service_id = (service_id + (10 - len(service_id)) * '\x00').encode('utf-8')
        # 计费用户类型字段Fee_UserType，1字节。
        # 0：对目的终端MSISDN计费
        # 1：对源终端MSISDN计费
        # 2：对SP计费
        # 3：表示本字段无效，对谁计费参见Fee_terminal_Id字段
        _fee_usertype = struct.pack('!B', fee_usertype)
        # 被计费用户的号码Fee_terminal_Id（如本字节填空，则表示本字段无效，对谁计费参见Fee_UserType字段，本字段与Fee_UserType字段互斥）。v2:21字节，v3:32字节
        if version == CMPP_VERSION['TWO']:
            _fee_terminal_id = (fee_terminal_id + (21 - len(fee_terminal_id)) * '\x00').encode('utf-8')
        else:
            _fee_terminal_id = (fee_terminal_id + (32 - len(fee_terminal_id)) * '\x00').encode('utf-8')
            # v3独有参数：被计费用户的号码类型，0：真实号码；1：伪码。1字节无符号整数
            self._fee_terminal_type = struct.pack('!B', fee_terminal_type)
        # GSM协议类型TP_pId。1字节。
        _tp_pid = struct.pack('!B', tp_pid)
        # GSM协议类型TP_udhi。1字节。长短信拆分时为1，普通为0
        if split_send:
            tp_udhi = 1
        _tp_udhi = struct.pack('!B', tp_udhi)
        # 信息格式Msg_Fmt。1字节。
        # 0：ASCII串
        # 3：短信写卡操作
        # 4：二进制信息
        # 8：UCS2编码
        # 15：含GB汉字
        _msg_fmt = struct.pack('!B', msg_fmt)
        # 信息内容来源(SP_Id)Msg_src。6字节。
        _msg_src = msg_src.encode('utf-8')
        # 资费类别FeeType。2字节。
        # 01：对“计费用户号码”免费
        # 02：对“计费用户号码”按条计信息费
        # 03：对“计费用户号码”按包月收取信息费
        # 04：对“计费用户号码”的信息费封顶
        # 05：对“计费用户号码”的收费是由SP实现
        _feetype = feetype.encode('utf-8')
        # 资费代码（以分为单位）FeeCode。6字节。
        _feecode = feecode.encode('utf-8')
        # 存活有效期ValId_Time。17字节。
        _valid_time = valid_time.encode('utf-8')
        # 定时发送时间At_Time。17字节。
        _at_time = at_time.encode('utf-8')
        # 源号码Src_Id。21字节。
        # SP的服务代码或前缀为服务代码的长号码, 网关将该号码完整的填到SMPP协议Submit_SM消息相应的source_addr字段，该号码最终在用户手机上显示为短消息的主叫号码。
        _src_id = (src_id + (21 - len(src_id)) * '\x00').encode('utf-8')
        # 接收信息的用户数量(小于100个用户)DestUsr_tl。1字节。
        _destusr_tl = struct.pack('!B', len(dest_terminal_id))
        # 接收短信的MSISDN号码Dest_terminal_Id。长度21*DestUsr_tl。
        _dest_terminal_id = b""
        if version == CMPP_VERSION['TWO']:
            for msisdn in dest_terminal_id:
                _dest_terminal_id += (msisdn + (21 - len(msisdn)) * '\x00').encode('utf-8')
        else:
            for msisdn in dest_terminal_id:
                _dest_terminal_id += (msisdn + (32 - len(msisdn)) * '\x00').encode('utf-8')
            # v3独有参数：接收短信的用户的号码类型，0：真实号码；1：伪码。1字节无符号整数
            self._dest_terminal_type = struct.pack('!B', dest_terminal_type)
        # 信息内容Msg_Content。长度不固定。
        if split_send:
            _msg_content = content_prefix + msg_content.encode('utf-16-be')
        else:
            _msg_content = msg_content.encode('utf-16-be')
        # 信息长度(Msg_Fmt值为0时：<160个字节；其它<=140个字节)Msg_Length。1字节。
        _msg_length = struct.pack('!B', len(_msg_content))
        if version == CMPP_VERSION['TWO']:
            # v2独有参数：保留Reserve。8字节。
            self._reserve = (reserve + (8 - len(reserve)) * '\x00').encode('utf-8')
        else:
            # v3独有参数：点播业务使用的LinkID，非点播类业务的MT流程不使用该字段。长度20的十六进制字节序列。
            self._link_id = (link_id + (20 - len(link_id)) * '\x00').encode('utf-8')

        # print(_msg_id.hex())
        # print(_pk_total.hex())
        # print(_pk_number.hex())
        # print(_registered_delivery.hex())
        # print(_service_id.hex())
        # print(_fee_usertype.hex())
        # print(_fee_terminal_id.hex())
        # print(_tp_pid.hex())
        # print(_tp_udhi.hex())
        # print(_msg_fmt.hex())
        # print(_msg_src.hex())
        # print(_feetype.hex())
        # print(_feecode.hex())
        # print(_valid_time.hex())
        # print(_at_time.hex())
        # print(_src_id.hex())
        # print(_destusr_tl.hex())
        # print(_dest_terminal_id.hex())
        # print(_msg_length.hex())
        # print(_msg_content.hex())
        # print(_reserve.hex())
        # 组装消息体
        _message_body = ''
        if version == CMPP_VERSION['TWO']:
            _message_body = _msg_id + \
                            _pk_total + _pk_number + \
                            _registered_delivery + \
                            _msg_level + \
                            _service_id + \
                            _fee_usertype + _fee_terminal_id + \
                            _tp_pid + _tp_udhi + \
                            _msg_fmt + _msg_src + \
                            _feetype + _feecode + \
                            _valid_time + _at_time + \
                            _src_id + _destusr_tl + \
                            _dest_terminal_id + \
                            _msg_length + \
                            _msg_content + \
                            self._reserve
        else:
            _message_body = _msg_id + \
                            _pk_total + _pk_number + \
                            _registered_delivery + \
                            _msg_level + \
                            _service_id + \
                            _fee_usertype + _fee_terminal_id + \
                            self._fee_terminal_type + \
                            _tp_pid + _tp_udhi + \
                            _msg_fmt + _msg_src + \
                            _feetype + _feecode + \
                            _valid_time + _at_time + \
                            _src_id + _destusr_tl + \
                            _dest_terminal_id + self._dest_terminal_type + \
                            _msg_length + \
                            _msg_content + \
                            self._link_id

        RequestData.__init__(self, CMPP_COMMAND_ID['CMPP_SUBMIT'], _message_body)



"""
socket封装，cmpp专用
"""
import socket
import struct
import time

from zpystream import Stream

from smssdk.tcp import callback_success, callback_failed
from smssdk.tcp.cmpp.callback import BasicCallback
from smssdk.tcp.cmpp.resdata.router import parse_response
from smssdk.utils.exceptions import SMSServiceException
from smssdk.utils.logger import Logger
from smssdk.utils.values import CMPP_COMMAND_ID, CMPP_CONNECT_RESP_STATUS, CMPP_SUBMIT_RESP_RESULT_V2, \
    CMPP_SUBMIT_RESP_RESULT_V3, CMPP_VERSION, NUMBERS, REPORT_STAT, MAX_REPORT_WAIT_TIME

logger = Logger('tcp.cmpp.sockettool.__init__.py')


class Socket:
    def __init__(self, cmpp_version: int, host: str, port: int, callback: BasicCallback):
        """
        初始化socket参数
        :param cmpp_version: CMPP版本。2/3
        :param host: 地址
        :param port: 端口
        :param callback: 回调实现
        """
        self._version = cmpp_version
        self._host = host
        self._port = port
        self._callback = callback
        self._so = None
        # 记录长短信发送数据
        self._split_send_rec = {}
        # 所有长短新拆分消息的发送序列号和对应响应的msg_id。结构：{seq_id: res_msg_id, ...}
        self._split_seq_and_res_msg_id = {}
        # 暂存需要从_split_send_rec中删除的元素的key
        self._need_del_key = []
        # 暂存需要从_split_seq_and_res_msg_id中删除的元素的key
        self._need_del_msg_id_key = []

    def connect_socket(self):
        """
        连接socket服务
        """
        try:
            self._so = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self._so.connect((self._host, self._port))
            logger.info(f'[Socket.connect_socket()][v{self._version}][connect]socket连接成功。')
        except Exception as e:
            logger.exception(e,
                             f'[Socket._connect_socket()][v{self._version}]socket服务连接失败，'
                             f'host:{self._host}，port:{self._port}。')
            raise SMSServiceException(
                f'[tcp.cmpp.sockettool.__init__.py:Socket._connect_socket()][v{self._version}]socket服务连接失败，'
                f'host:{self._host}，port:{self._port}。')

    def close_socket(self):
        """
        断开socket服务
        """
        try:
            self._so.close()
            logger.info(f'[Socket.close_socket()][v{self._version}][close]socket已断开连接。')
        except Exception as e:
            logger.exception(e, '[Socket._close_socket()][v{self._version}]断开socket连接失败。')
            raise SMSServiceException(
                '[tcp.cmpp.sockettool.__init__.py:Socket._close_socket()][v{self._version}]断开socket连接失败。')

    def write(self, message, event):
        """
        socket客户端发送消息
        """
        try:
            self._so.send(message)
            logger.info(f'[Socket.write()][{event}][v{self._version}][send]{message.hex()}。')
        except Exception as e:
            logger.exception(e,
                             f'[Socket.write()][{event}][v{self._version}][send]socket send message failed，message:{message.hex()}')
            raise SMSServiceException(
                f'[tcp.cmpp.sockettool.__init__.py:Socket.write()][{event}][v{self._version}][send]'
                f'socket send message failed，message:{message.hex()}')

    def read(self):
        """
        持续监听服务端的消息
        """
        while 1:
            if getattr(self._so, '_closed'):
                logger.info(f'[Socket.read()]socket连接已关闭，读取服务端消息线程即将结束。')
                return
            response = None
            try:
                content_length = self._so.recv(4)
                if content_length:
                    length, = struct.unpack('!L', content_length)
                    response = content_length + self._so.recv(length - 4)
                    logger.info(f'[Socket.read()][recv]'
                                f'length:{length}，content:{response.hex()}。')
            except ConnectionAbortedError:
                logger.info(f'[Socket.read()]socket连接已关闭/断开，读取服务端消息线程即将结束。')
                return
            # 不同事件做不同处理
            parsed_res = parse_response(response, self._version)
            # 登录/连接ISMG响应
            if parsed_res.command_id == CMPP_COMMAND_ID['CMPP_CONNECT_RESP']:
                if parsed_res.status == CMPP_CONNECT_RESP_STATUS['SUCCESS']['code']:
                    self._callback.for_connect(callback_success())
                else:
                    ismg_res_status = parsed_res.status
                    failed_item = Stream(CMPP_CONNECT_RESP_STATUS.keys()).filter(
                        lambda x: CMPP_CONNECT_RESP_STATUS[x]['code'] == ismg_res_status).to_list()
                    if failed_item and len(failed_item) > 0:
                        self._callback.for_connect(callback_failed(CMPP_CONNECT_RESP_STATUS[failed_item[0]]))
                    else:
                        logger.warning(f'[Socket.read()][v{self._version}][CMPP_CONNECT_RESP]事件status异常，不在任何枚举值之中。')
                        self._callback.for_connect(callback_failed({}))
                    logger.info(f'[Socket.read()][v{self._version}][CMPP_CONNECT_RESP]事件异常，即将断开socket连接。')
                    self.close_socket()
            # 向ISMG提交消息事件响应
            elif parsed_res.command_id == CMPP_COMMAND_ID['CMPP_SUBMIT_RESP']:
                res_code = None
                if self._version == CMPP_VERSION['TWO']:
                    res_code = CMPP_SUBMIT_RESP_RESULT_V2
                else:
                    res_code = CMPP_SUBMIT_RESP_RESULT_V3
                # 是响应的长短信拆分的消息，找出长短信发送记录，做相关处理
                if len(self._split_seq_and_res_msg_id) > NUMBERS['ZERO'] and \
                        parsed_res.sequence in self._split_seq_and_res_msg_id.keys():
                    for k1, v1 in self._split_send_rec.items():
                        for k2, v2 in v1['message'].items():
                            if v2['seq_id'] == parsed_res.sequence:
                                self._split_seq_and_res_msg_id[v2['seq_id']] = parsed_res.msg_id
                                # 跳过已无效的记录
                                if v1['invalid']:
                                    logger.info(f'[Socket.read()][v{self._version}][CMPP_SUBMIT_RESP]'
                                                f'已跳过一条响应，因该条响应所在的拆分短信组已被置为无效，'
                                                f'seq:{parsed_res.sequence}，msg_id:{parsed_res.msg_id}。')
                                    break
                                # 提取该组seq_id
                                all_seq_ids = []
                                for k3, v3 in v1['message'].items():
                                    all_seq_ids.append(v3['seq_id'])
                                # 该条短信已收到响应
                                v1['message'][k2]['received_res'] = True
                                if v2['seq_id'] == v1['main_seq_id']:
                                    v1['main_msg_id'] = parsed_res.msg_id
                                # 该条成功，做记录
                                if parsed_res.result == res_code['SUCCESS']['result']:
                                    v1['res_success_count'] += NUMBERS['ONE']
                                    v1['message'][k2]['service_success'] = True
                                    # 该组拆分短信的所有响应接收完毕，做成功回调
                                    if v1['total_count'] == v1['res_success_count']:
                                        logger.info(
                                            f"[Socket.read()][v{self._version}][CMPP_SUBMIT_RESP][split]"
                                            f"第{k1}组长短信拆分业务响应正常，该组对应的拆分短信sequence分别为：{all_seq_ids}，"
                                            f"主msg_id为{v1['main_msg_id']}。即将发出成功回调。")
                                        self._callback.for_submit(callback_success({'msg_id': v1['main_msg_id']}))
                                # 该条异常，则所有失败，做失败回调，并删除该组记录
                                else:
                                    logger.info(
                                        f"[Socket.read()][v{self._version}][CMPP_SUBMIT_RESP][split]"
                                        f"第{k1}组长短信拆分业务失败，收到状态异常的响应，该组对应的拆分短信sequence分别为：{all_seq_ids}，"
                                        f"主msg_id为{v1['main_msg_id']}。即将发出失败回调。")
                                    ismg_res_result = parsed_res.result
                                    failed_item = Stream(res_code.keys()).filter(
                                        lambda x: res_code[x]['result'] == ismg_res_result).to_list()
                                    if failed_item and len(failed_item) > 0:
                                        self._callback.for_submit(callback_failed(
                                            {
                                                'msg_id': v1['main_msg_id'],
                                                'failed_res': res_code[failed_item[0]]
                                            }
                                        ))
                                    else:
                                        logger.warning(
                                            f'[Socket.read()][v{self._version}][CMPP_SUBMIT_RESP]['
                                            f'split]事件result异常，不在任何枚举值之中。')
                                        self._callback.for_submit(callback_failed({'msg_id': v1['main_msg_id']}))
                                    # 发生异常响应，该组记录置为过去式
                                    v1['invalid'] = True
                                    v1['invalid_timestamp'] = int(time.time())
                                    break
                    # 删除已invalid超过10分钟的拆分记录
                    self._del_invalid_records()
                # 普通短信发送响应
                else:
                    # 成功
                    if parsed_res.result == res_code['SUCCESS']['result']:
                        self._callback.for_submit(callback_success({'msg_id': parsed_res.msg_id}))
                    # 异常
                    else:
                        ismg_res_result = parsed_res.result
                        failed_item = Stream(res_code.keys()).filter(
                            lambda x: res_code[x]['result'] == ismg_res_result).to_list()
                        if failed_item and len(failed_item) > 0:
                            self._callback.for_submit(callback_failed(
                                {
                                    'msg_id': parsed_res.msg_id,
                                    'failed_res': res_code[failed_item[0]]
                                }
                            ))
                        else:
                            logger.warning(f'[Socket.read()][v{self._version}][CMPP_SUBMIT_RESP]事件result异常，不在任何枚举值之中。'
                                           f'msg_id:{parsed_res.msg_id}。')
                            self._callback.for_submit(callback_failed({'msg_id': parsed_res.msg_id}))
            # 向ISMG提交信息后，ISMG主动发送的状态报告
            elif parsed_res.command_id == CMPP_COMMAND_ID['CMPP_DELIVER']:
                if parsed_res.is_report:
                    # 是否是拆分短信对应的状态报告（规则：submit res msg id = report send msg id），如果是，做相关记录赋值及整合判断。
                    if len(self._split_seq_and_res_msg_id) > NUMBERS['ZERO'] and \
                            parsed_res.msg_id in self._split_seq_and_res_msg_id.values():
                        send_seq_id = None
                        for k0, v0 in self._split_seq_and_res_msg_id.items():
                            if v0 == parsed_res.msg_id:
                                send_seq_id = k0
                        for k1, v1 in self._split_send_rec.items():
                            for k2, v2 in v1['message'].items():
                                if v2['seq_id'] == send_seq_id:
                                    # 跳过已无效的记录
                                    if v1['invalid']:
                                        logger.info(f'[Socket.read()][v{self._version}][CMPP_DELIVER]'
                                                    f'已跳过一条状态报告，因该报告对应所在的拆分短信组已被置为无效，'
                                                    f'seq:{parsed_res.sequence}，msg_id:{parsed_res.msg_id}。')
                                        continue
                                    # 提取所有该组所有seq_id
                                    all_seq_ids = []
                                    for k3, v3 in v1['message'].items():
                                        all_seq_ids.append(v3['seq_id'])
                                    # 更新该组记录最近接收状态报告的时间
                                    v1['last_report_timestamp'] = int(time.time())
                                    # 该条状态报告成功（stat为DELIVRD）
                                    if parsed_res.callback_res['stat'] == REPORT_STAT['DELIVRD']['FINAL_MESSAGE_STATE']:
                                        v1['reported_success_count'] += NUMBERS['ONE']
                                        v1['all_report'].append(parsed_res.callback_res)
                                        # 如果收到stat为DELIVRD的报告数量达到总拆分条数，说明全部成功，即整个长短信业务成功
                                        if v1['reported_success_count'] == v1['total_count']:
                                            logger.info(
                                                f"[Socket.read()][v{self._version}][CMPP_DELIVER][split]"
                                                f"第{k1}组长短信拆分业务状态报告正常，该组对应的拆分短信sequence分别为：{all_seq_ids}，"
                                                f"主msg_id为{v1['main_msg_id']}。即将发出成功回调。")
                                            self._callback.for_submit_report(callback_success(
                                                {
                                                    'msg_id': v1['main_msg_id'],
                                                    'all_report': v1['all_report']
                                                }
                                            ))
                                            # 该组拆分消息的状态报告接收完毕，设为过去式
                                            v1['invalid'] = True
                                            v1['invalid_timestamp'] = int(time.time())
                                    # 该条状态报告异常（stat为DELIVRD）
                                    else:
                                        logger.info(
                                            f"[Socket.read()][v{self._version}][CMPP_DELIVER][split]"
                                            f"第{k1}组长短信拆分业务状态报告异常，收到状态stat非DELIVRD的状态报告，"
                                            f"该组对应的拆分短信sequence分别为：{all_seq_ids}，"
                                            f"msg_id:{v1['main_msg_id']}。即将发出失败回调。")
                                        v1['all_report'].append(parsed_res.callback_res)
                                        self._callback.for_submit_report(callback_failed(
                                            {
                                                'msg_id': v1['main_msg_id'],
                                                'all_report': v1['all_report']
                                            }
                                        ))
                                        # 该组消息拆分记录存在异常状态报告，整组记录设为过去式
                                        v1['invalid'] = True
                                        v1['invalid_timestamp'] = int(time.time())
                                    break
                        # 检查删除已invalid5分钟的记录
                        self._del_invalid_records()
                        # 顺带检查是否有异常记录，有的话删了
                        now_timestamp = int(time.time())
                        need_del_key_by_time = []
                        for k, v in self._split_send_rec.items():
                            if now_timestamp - v['last_report_timestamp'] > MAX_REPORT_WAIT_TIME:
                                need_del_key_by_time.append(k)
                        if need_del_key_by_time:
                            self._del_invalid_records(need_del_key_by_time)
                    # 普通短信的状态报告
                    else:
                        if parsed_res.callback_res['stat'] == REPORT_STAT['DELIVRD']['FINAL_MESSAGE_STATE']:
                            self._callback.for_submit_report(callback_success(
                                {
                                    'msg_id': parsed_res.msg_id,
                                    'all_report': [parsed_res.callback_res]
                                }
                            ))
                        else:
                            logger.info(
                                f"[Socket.read()][v{self._version}][CMPP_DELIVER][split]"
                                f"普通短信状态报告异常，收到状态stat非DELIVRD的状态报告，"
                                f"msg_id:{parsed_res.msg_id}。即将发出失败回调。")
                            self._callback.for_submit_report(callback_failed(
                                {
                                    'msg_id': parsed_res.msg_id,
                                    'all_report': [parsed_res.callback_res]
                                }
                            ))

            # 激活测试/心跳
            elif parsed_res.command_id == CMPP_COMMAND_ID['CMPP_ACTIVE_TEST']:
                # logger.info(f'[Socket.read()][v{self._version}]收到激活测试。')
                pass

    def is_online(self):
        """
        socket连接是还处于连接状态
        :return: True是，False否
        """
        return self._so and not getattr(self._so, '_closed')

    def registry_split_rec(self, k: int, v: dict):
        """
        注册新的长短新拆分发送记录、并提取消息序列号
        :param k: 该组记录的key
        :param v: 该组记录的数据
        """
        self._split_send_rec[k] = v
        for k1, v1 in v['message'].items():
            self._split_seq_and_res_msg_id[v1['seq_id']] = None

    def _del_key_from_registry_split_rec(self, k: list):
        """
        从拆分消息记录中删除若干组数据
        :param k: keys
        """
        if not k:
            return
        for x in k:
            del self._split_send_rec[x]

    def _del_key_from_seq_and_res_msg_id(self, k: list):
        """
        从seq_id:msg_id记录中删除若干组数据
        :param k: keys
        """
        if not k:
            return
        for x in k:
            del self._split_seq_and_res_msg_id[x]

    def _del_invalid_records(self, a_k: list = None):
        """
        传递k时，从拆分记录中删除key为k的记录及可能存在的关联seq_id:msg_id记录；
        不传递k时，检索全部拆分数据记录，从中删除invalid已超过5分钟的元素以及其关联的seq_id:msg_id记录
        :return:
        """
        if not a_k:
            all_record_key = []
            all_seq_ids_del = []
            for k, v in self._split_send_rec.items():
                if v['invalid'] and int(time.time()) - v['invalid_timestamp'] > NUMBERS['THREE_HUNDREDS']:
                    all_record_key.append(k)
                    for k1, v1 in v['message'].items():
                        if v1['seq_id'] in self._split_seq_and_res_msg_id:
                            all_seq_ids_del.append(v1['seq_id'])
            if all_seq_ids_del:
                self._del_key_from_seq_and_res_msg_id(all_seq_ids_del)
            if all_record_key:
                self._del_key_from_registry_split_rec(a_k)
        else:
            for x in a_k:
                v = self._split_send_rec[x]
                all_seq_ids_del = []
                for k1, v1 in v['message'].items():
                    if v1['seq_id'] in self._split_seq_and_res_msg_id:
                        all_seq_ids_del.append(v1['seq_id'])
                if all_seq_ids_del:
                    self._del_key_from_seq_and_res_msg_id(all_seq_ids_del)
            self._del_key_from_registry_split_rec(a_k)



"""
解析ISMG->SP激活测试
"""
from smssdk.tcp.cmpp.resdata import ResponseData
from smssdk.utils.logger import Logger

logger = Logger('tcp.cmpp.resdata.active_res.py')


class ActiveReq(ResponseData):
    def __init__(self, message):
        # logger.info(f'[ActiveRes.__init__()]收到心跳，消息参数：{message.hex()}。')

        # 是心跳
        # self.is_active_test = True

        # 无消息体
        ResponseData.__init__(self, message)



"""
一些常量
"""

""" 请求地址 """
URLS = {
    'TOKEN': 'https://gateway.dev.codebetterlife.com/api/v2/auth',
    'BATCH_SMS': 'https://gateway.dev.codebetterlife.com/api/v2/send',
    'P2P_SMS': 'https://gateway.dev.codebetterlife.com/api/v2/p2p'
}

""" 后台统一http响应相关信息 """
RESPONSE_INFO = {
    'CODE_OBJ': 'resultCode',
    'DATA': 'data',
    'RESPONSE_CODE': 'code',
    'RESPONSE_MESSAGE': 'message',
    'SUCCESS_CODE': 0,
    'AUTH_NAME': 'Authorization'
}

""" 后台api固定headers """
STABLE_HEADERS = {
    'CONTENT_TYPE': 'application/json;charset=utf-8'
}

""" 正则 """
REGEX = {
    'MOBILE_PHONE': '^1(3\\d|4[5-9]|5[0-35-9]|6[567]|7[0-8]|8\\d|9[0-35-9])\\d{8}$'
}

""" CMPP协议command_id """
CMPP_COMMAND_ID = {
    # SP->ISMG连接
    'CMPP_CONNECT': 0x00000001,
    # SP->ISMG断开连接
    'CMPP_TERMINATE': 0x00000002,
    # SP->ISMG发送信息
    'CMPP_SUBMIT': 0x00000004,
    # SP->ISMG连接应答
    'CMPP_CONNECT_RESP': 0x80000001,
    # SP->ISMG断开连接应答
    'CMPP_TERMINATE_RESP': 0x80000002,
    # SP->ISMG发送信息应答
    'CMPP_SUBMIT_RESP': 0x80000004,
    # ISMG->SP发送信息
    'CMPP_DELIVER': 0x00000005,
    # 激活测试
    'CMPP_ACTIVE_TEST': 0x00000008
}

""" SP->ISMG:CMPP_CONNECT_RESP status """
CMPP_CONNECT_RESP_STATUS = {
    # 成功
    'SUCCESS': {
        'code': 0,
        'meaning': '成功'
    },
    # 消息结构有误
    'MSG_FORMAT_ERROR': {
        'code': 1,
        'meaning': '消息结构有误'
    },
    # 非法源地址
    'INVALID_SOURCE_ADDR': {
        'code': 2,
        'meaning': '非法源地址'
    },
    # 认证错误
    'AUTH_FAILED': {
        'code': 3,
        'meaning': '认证错误'
    },
    # 版本太高
    'VERSION_TOO_HIGH': {
        'code': 4,
        'meaning': '版本太高'
    },
    # 其它错误
    'OTHER_ERROR': {
        'code': 5,
        'meaning': '其它错误'
    },
}

""" SP->ISMG:CMPP_SUBMIT_RESP v2.0 result """
CMPP_SUBMIT_RESP_RESULT_V2 = {
    # 正确
    'SUCCESS': {
        'result': 0,
        'meaning': '正确'
    },
    # 消息结构有误
    'MSG_FORMAT_ERROR': {
        'result': 1,
        'meaning': '消息结构有误'
    },
    # 命令字错
    'COMMAND_ERROR': {
        'result': 2,
        'meaning': '命令字错'
    },
    # 消息序号重复
    'MSG_NUMBER_DUPLICATE': {
        'result': 3,
        'meaning': '消息序号重复'
    },
    # 消息长度错误
    'MSG_LENGTH_PARAM_ERROR': {
        'result': 4,
        'meaning': '消息长度错误'
    },
    # 资费代码错误
    'FEE_CODE_ERROR': {
        'result': 5,
        'meaning': '资费错误'
    },
    # 超过最大信息长度限制
    'MSG_LENGTH_EXCEED': {
        'result': 6,
        'meaning': '超过最大信息长度限制'
    },
    # 业务代码错
    'SERVICE_PROGRAM_ERROR': {
        'result': 7,
        'meaning': '业务代码错'
    },
    # 流量控制错
    'FLOW_CONTROL_ERROR': {
        'result': 8,
        'meaning': '流量控制错'
    },
    # 其它错误
    'OTHER_ERROR': {
        'result': 9,
        'meaning': '其它错误'
    },
}

""" SP->ISMG:CMPP_SUBMIT_RESP v3.0 result """
CMPP_SUBMIT_RESP_RESULT_V3 = {
    # 正确
    'SUCCESS': {
        'result': 0,
        'meaning': '正确'
    },
    # 消息结构有误
    'MSG_FORMAT_ERROR': {
        'result': 1,
        'meaning': '消息结构有误'
    },
    # 命令字错
    'COMMAND_ERROR': {
        'result': 2,
        'meaning': '命令字错'
    },
    # 消息序号重复
    'MSG_NUMBER_DUPLICATE': {
        'result': 3,
        'meaning': '消息序号重复'
    },
    # 消息长度错误
    'MSG_LENGTH_PARAM_ERROR': {
        'result': 4,
        'meaning': '消息长度错误'
    },
    # 资费代码错误
    'FEE_CODE_ERROR': {
        'result': 5,
        'meaning': '资费错误'
    },
    # 超过最大信息长度限制
    'MSG_LENGTH_EXCEED': {
        'result': 6,
        'meaning': '超过最大信息长度限制'
    },
    # 业务代码错
    'SERVICE_PROGRAM_ERROR': {
        'result': 7,
        'meaning': '业务代码错'
    },
    # 流量控制错
    'FLOW_CONTROL_ERROR': {
        'result': 8,
        'meaning': '流量控制错'
    },
    # 本网关不负责服务此计费号码
    'OTHER_ERROR': {
        'result': 9,
        'meaning': '本网关不负责服务此计费号码'
    },
    # 本网关不负责服务此计费号码
    'SRC_ID_ERROR': {
        'result': 10,
        'meaning': 'Src_Id错误'
    },
    # Msg_src错误
    'MSG_SRC_ERROR': {
        'result': 11,
        'meaning': 'Msg_src错误'
    },
    # Fee_terminal_Id错误
    'FEE_TERMINAL_ID_ERROR': {
        'result': 12,
        'meaning': 'Fee_terminal_Id错误'
    },
    # Dest_terminal_Id错误
    'DEST_TERMINAL_ID': {
        'result': 13,
        'meaning': 'Dest_terminal_Id错误'
    }
}

""" 常用数字 """
NUMBERS = {
    'ZERO': 0,
    'ONE': 1,
    'MAX_PORT': 65535,
    'THREE_HUNDREDS': 300,
    'SEVENTY': 70,
    'SIXTY_SEVEN': 67
}

""" cmpp2.0/3.0协议中ISMG->SP消息是否为状态报告 """
CMPP_REGISTERED_DELIVERY = {
    'NOT_REPORT': 0,
    'IS_REPORT': 1
}

""" 回调响应码 """
CALLBACK_CODE = {
    'SUCCESS': {
        'code': 0,
        'message': '成功'
    },
    'FAILED': {
        'code': 1,
        'message': '异常'
    },
}

""" 支持的CMPP协议版本号 """
CMPP_VERSION = {
    'TWO': 2,
    'THREE': 3,
}

""" 长短信拆分标志前缀 """
MESSAGE_SPLIT_MARK_PREFIX = b'/x05/x00/x03'

""" ISMG->SP:CMPP_DELIVER事件中，状态报告里的stat """
REPORT_STAT = {
    'DELIVRD': {
        'MESSAGE_STATE': 'DELIVERED',
        'FINAL_MESSAGE_STATE': 'DELIVRD',
        'DESCRIPTION': 'Message is delivered to destination.'
    },
    'EXPIRED': {
        'MESSAGE_STATE': 'EXPIRED',
        'FINAL_MESSAGE_STATE': 'EXPIRED',
        'DESCRIPTION': 'Message validity period has expired.'
    },
    'DELETED': {
        'MESSAGE_STATE': 'DELETED',
        'FINAL_MESSAGE_STATE': 'DELETED',
        'DESCRIPTION': 'Message has been deleted.'
    },
    'UNDELIV': {
        'MESSAGE_STATE': 'UNDELIVERABLE',
        'FINAL_MESSAGE_STATE': 'UNDELIV',
        'DESCRIPTION': 'Message is undeliverable.'
    },
    'ACCEPTD': {
        'MESSAGE_STATE': 'ACCEPTED',
        'FINAL_MESSAGE_STATE': 'ACCEPTD',
        'DESCRIPTION': 'Message is in accepted state(i.e. has been manually read on behalf of the subscriber by '
                       'customer service). '
    },
    'UNKNOWN': {
        'MESSAGE_STATE': 'UNKNOWN',
        'FINAL_MESSAGE_STATE': 'UNKNOWN',
        'DESCRIPTION': 'Message is in invalid state.'
    },
    'REJECTD': {
        'MESSAGE_STATE': 'REJECTED',
        'FINAL_MESSAGE_STATE': 'REJECTD',
        'DESCRIPTION': 'Message is in a rejected state.'
    }
}

""" 长短信拆分时，接收状态报告最大等待时长。单位s """
MAX_REPORT_WAIT_TIME = 60



"""
自定义线程类
"""
import threading

from smssdk.utils.logger import Logger

logger = Logger('mythread.py')
threadLock = threading.Lock()


class Thread(threading.Thread):
    def __init__(self, target=None, args=(), kwargs=None):
        super().__init__()
        if kwargs is None:
            kwargs = {}
        self.target = target
        self.args = args
        self.kwargs = kwargs

    def run(self):
        self.target(*self.args, **self.kwargs)



from setuptools import (
    find_packages,
    setup,
)

setup(
    name='sms_sdk',
    version='0.0.1',
    description='sms sdk',
    classifiers=[],
    keywords='sms sdk',
    author='zgl',
    author_email='',
    url='',
    license='MIT',
    packages=find_packages(exclude=[]),
    package_data={'': ['*.*']},
    include_package_data=True,
    install_requires=[
        'colorama~=0.4.4',
        'requests~=2.28.1',
        'zpystream~=0.1.1'
    ],
    long_description='sms sdk by python'
)



"""
日志工具类封装
"""

import logging
from colorama import Fore, Style


class Logger(object):
    def __init__(self, module_name):
        self.logger = logging.getLogger(name=module_name)
        self.logger.setLevel(logging.INFO)
        # 日志输出格式
        self.formatter = logging.Formatter("[%(asctime)s] [" + module_name + "] [%(levelname)s] %(message)s")

        # 解决被父级调用时输出重复的问题（每次输出会多一份无格式的默认样式日志）
        self.logger.parent = None

    def __console(self, level, message, e):

        # 创建一个StreamHandler,用于输出到控制台
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        ch.setFormatter(self.formatter)
        self.logger.addHandler(ch)

        if level == "info":
            self.logger.info(Fore.WHITE + str(message) + Style.RESET_ALL)
        elif level == "debug":
            self.logger.debug(Fore.WHITE + str(message) + Style.RESET_ALL)
        elif level == "warning":
            self.logger.warning(Fore.YELLOW + str(message) + Style.RESET_ALL)
        elif level == "error":
            self.logger.error(Fore.RED + str(message) + Style.RESET_ALL)
        elif level == "exception":
            self.logger.exception(e)

        # 避免重复输出
        self.logger.removeHandler(ch)

    def debug(self, message):
        self.__console("debug", message, None)

    def info(self, message):
        self.__console("info", message, None)

    def warning(self, message):
        self.__console("warning", message, None)

    def error(self, message):
        self.__console("error", message, None)

    def exception(self, e, message=None):
        self.__console("exception", message, e)



"""
封装tcp请求数据
"""

import struct
from smssdk.utils.values import NUMBERS

# 消息流水号,顺序累加,步长为1
_global_seq_no = NUMBERS['ZERO']


class RequestData:
    def __init__(self, command_id, message_body):
        """
        :param command_id: CMPP命令代码，如0x00000001
        :param message_body: 这次请求的消息体
        """
        self._command_id = command_id
        self._message_body = message_body

        global _global_seq_no
        # 步长为1
        _global_seq_no += NUMBERS['ONE']
        self.sequence = _global_seq_no

        # 拼接最终请求参数(消息头+消息体)。!L:long型格式，存储时按大端排序
        # 格式：消息头(消息总长度(含该值的长度)+命令id+消息流水号) + 消息体
        # 对应长度：12(4+4+4) + len(消息体)
        self.message = struct.pack('!L', 12 + len(self._message_body)) \
                       + struct.pack('!L', self._command_id) \
                       + struct.pack('!L', self.sequence) \
                       + self._message_body



"""
ISMG响应/ISMG主动事件回调
"""
from smssdk.utils.logger import Logger

logger = Logger('tcp.cmpp.callback.__init__.py')


class BasicCallback:
    def for_connect(self, res: dict):
        """
        连接ISMG回调
        :param res: 响应数据，结构如下：
        {
            'code': 0为成功，1为异常：int
            'message': code相应的说明：string
            'data': 成功时内容为None，异常时该key包含具体原因，内容同CMPP协议规范中返回内容：dict
        }
        """
        logger.info(f'[BasicCallback:for_connect()][CMPP_CONNECT回调]{res}。')

    def for_submit(self, res: dict):
        """
        SP向ISMG发送消息（发短信）回调
        :param res: 响应数据，结构如下：
        {
            'code': 0为成功，1为异常：int
            'message': code相应的说明：string
            'data': {
                'msg_id': 响应msg_id。长短信拆分时，为代表该组拆分短信的响应中的主msg_id：string
                'failed_res': 失败时，提供原始响应数据：list[dict]
            }
        }
        """
        logger.info(f'[BasicCallback:for_submit()][CMPP_SUBMIT回调]{res}。')

    def for_submit_report(self, res: dict):
        """
        收到ISMG发送的状态报告的回调
        :param res: 响应数据，结构如下：
        {
            'code': 0为成功，1为异常：int
            'message': code相应的说明：string
            'data': {
                'msg_id': 报告msg_id。长短信拆分时，为代表该组拆分短信的状态报告中的主msg_id：string
                'all_report': 状态报告原始详细信息。长短信拆分时，包含多条，若失败，失败的状态报告也在其中：list[dict]
            }
        }
        """
        logger.info(f'[BasicCallback:for_submit_report()][CMPP_DELIVER(REPORT)回调]{res}。')



"""
SP->ISMG连接请求参数封装
"""
import hashlib
import struct
import time
from smssdk.utils.values import CMPP_COMMAND_ID, CMPP_VERSION

from smssdk.tcp.cmpp.reqdata import RequestData


class ConnectReq(RequestData):
    def __init__(self, sp_id, sp_secret, version: int):
        """
        封装SP->ISMG连接请求的消息体
        :version: CMPP版本。2/3
        """
        # 源地址Source_Addr。6字节。
        _sp_id = sp_id.encode('utf-8')
        # SP公钥。16字节。
        _sp_secret = sp_secret.encode("utf-8")
        # CMPP协议版本号Version（!B：格式为integer，存储时按大端排列；0x20：0010 0000）。1字节。
        if version == CMPP_VERSION['TWO']:
            _version = struct.pack('!B', 0x20)
        else:
            _version = struct.pack('!B', 0x30)
        _time_str = time.strftime('%m%d%H%M%S', time.localtime(time.time()))
        # 时间戳Timestamp（!L：格式为long，存储时按大端排列）。4字节。
        _timestamp = struct.pack('!L', int(_time_str))

        # 封装鉴别源地址AuthenticatorSource。（b'\x00'：0的字节类型）
        # 加密规则：MD5（Source_Addr + 9字节的0 + shared secret + timestamp）。timestamp格式：MMDDHHMMSS，即月日时分秒，10位。
        self.authenticator_source = hashlib.md5(_sp_id + 9 * b'\x00' + _sp_secret + _time_str.encode('utf-8')).digest()

        # 组装消息体
        message_body = _sp_id + self.authenticator_source + _version + _timestamp

        # 组装最终消息=消息头+消息体
        RequestData.__init__(self, CMPP_COMMAND_ID['CMPP_CONNECT'], message_body)
